package com.sun.xml.internal.ws.addressing.policy;

import com.sun.xml.internal.ws.policy.NestedPolicy;
import java.util.Iterator;
import com.sun.xml.internal.ws.policy.Policy;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.resources.ModelerMessages;
import com.sun.xml.internal.bind.util.Which;
import com.sun.xml.internal.ws.policy.PolicyException;
import javax.xml.ws.soap.AddressingFeature;
import com.sun.xml.internal.ws.addressing.W3CAddressingMetadataConstants;
import java.util.logging.Level;
import com.sun.xml.internal.ws.api.addressing.AddressingVersion;
import com.sun.xml.internal.ws.policy.PolicyAssertion;
import com.sun.xml.internal.ws.policy.AssertionSet;
import java.util.LinkedList;
import javax.xml.ws.WebServiceFeature;
import java.util.Collection;
import com.sun.xml.internal.ws.policy.PolicyMap;
import com.sun.xml.internal.ws.policy.PolicyMapKey;
import javax.xml.namespace.QName;
import com.sun.xml.internal.ws.policy.privateutil.PolicyLogger;
import com.sun.xml.internal.ws.policy.jaxws.spi.PolicyFeatureConfigurator;

public class AddressingFeatureConfigurator implements PolicyFeatureConfigurator
{
    private static final PolicyLogger LOGGER;
    private static final QName[] ADDRESSING_ASSERTIONS;
    
    public AddressingFeatureConfigurator() {
    }
    
    public Collection<WebServiceFeature> getFeatures(PolicyMapKey key, PolicyMap policyMap) throws PolicyException {
        AddressingFeatureConfigurator.LOGGER.entering(new Object[] { key, policyMap });
        Collection<WebServiceFeature> features = new LinkedList();
        if (key != null && policyMap != null) {
            Policy policy = policyMap.getEndpointEffectivePolicy(key);
            for (QName addressingAssertionQName : AddressingFeatureConfigurator.ADDRESSING_ASSERTIONS) {
                if (policy != null && policy.contains(addressingAssertionQName)) {
                    for (AssertionSet assertionSet : policy) {
                        for (PolicyAssertion assertion : assertionSet) {
                            if (assertion.getName().equals(addressingAssertionQName)) {
                                WebServiceFeature feature = AddressingVersion.getFeature(addressingAssertionQName.getNamespaceURI(), true, !assertion.isOptional());
                                if (AddressingFeatureConfigurator.LOGGER.isLoggable(Level.FINE)) {
                                    AddressingFeatureConfigurator.LOGGER.fine("Added addressing feature \"" + feature + "\" for element \"" + key + "\"");
                                }
                                features.add(feature);
                            }
                            else {
                                continue;
                            }
                        }
                    }
                }
            }
            if (policy != null && policy.contains(W3CAddressingMetadataConstants.WSAM_ADDRESSING_ASSERTION)) {
                for (AssertionSet assertions2 : policy) {
                    for (PolicyAssertion assertion2 : assertions2) {
                        if (assertion2.getName().equals(W3CAddressingMetadataConstants.WSAM_ADDRESSING_ASSERTION)) {
                            NestedPolicy nestedPolicy = assertion2.getNestedPolicy();
                            boolean requiresAnonymousResponses = false;
                            boolean requiresNonAnonymousResponses = false;
                            if (nestedPolicy != null) {
                                requiresAnonymousResponses = nestedPolicy.contains(W3CAddressingMetadataConstants.WSAM_ANONYMOUS_NESTED_ASSERTION);
                                requiresNonAnonymousResponses = nestedPolicy.contains(W3CAddressingMetadataConstants.WSAM_NONANONYMOUS_NESTED_ASSERTION);
                            }
                            if (requiresAnonymousResponses && requiresNonAnonymousResponses) {
                                throw new WebServiceException("Only one among AnonymousResponses and NonAnonymousResponses can be nested in an Addressing assertion");
                            }
                            WebServiceFeature feature2;
                            try {
                                if (requiresAnonymousResponses) {
                                    feature2 = new AddressingFeature(true, !assertion2.isOptional(), AddressingFeature.Responses.ANONYMOUS);
                                }
                                else if (requiresNonAnonymousResponses) {
                                    feature2 = new AddressingFeature(true, !assertion2.isOptional(), AddressingFeature.Responses.NON_ANONYMOUS);
                                }
                                else {
                                    feature2 = new AddressingFeature(true, !assertion2.isOptional());
                                }
                            }
                            catch (NoSuchMethodError e) {
                                throw (PolicyException)AddressingFeatureConfigurator.LOGGER.logSevereException(new PolicyException(ModelerMessages.RUNTIME_MODELER_ADDRESSING_RESPONSES_NOSUCHMETHOD(toJar(Which.which(AddressingFeature.class))), e));
                            }
                            if (AddressingFeatureConfigurator.LOGGER.isLoggable(Level.FINE)) {
                                AddressingFeatureConfigurator.LOGGER.fine("Added addressing feature \"" + feature2 + "\" for element \"" + key + "\"");
                            }
                            features.add(feature2);
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        AddressingFeatureConfigurator.LOGGER.exiting(features);
        return features;
    }
    
    private static String toJar(String url) {
        if (!url.startsWith("jar:")) {
            return url;
        }
        url = url.substring(4);
        return url.substring(0, url.lastIndexOf(33));
    }
    
    static {
        LOGGER = PolicyLogger.getLogger(AddressingFeatureConfigurator.class);
        ADDRESSING_ASSERTIONS = new QName[] { new QName(AddressingVersion.MEMBER.policyNsUri, "UsingAddressing") };
    }
}
